# -*- perl -*-
use lib qw(t lib);
use strict;
use Test;
use ConfigSpec;
use ConfigSpec3;

plan(tests => 2);

my $c = new ConfigSpec;
my $c3 = new ConfigSpec3;

ok($c->canonical_lexicon,
   q{{"core" => {"section" => {"base" => {"default" => "null","mandatory" => 1},"number" => {"array" => 1,"re" => "^\\\\d+\\$"},"size" => {"re" => "\\\\d+(?:(?i) *[kmg])"}}},"load" => {"section" => {"*" => {},"file" => {"check" => "_check_abs_name","mandatory" => 1}}}}});

ok($c3->canonical_lexicon,
   q{{"core" => {"section" => {"root" => {"mandatory" => 1}}},"dir" => {"section" => {"diag" => {},"store" => {},"temp" => {}}}}});



